"""A TUI app to watch youtube videos of suscribed channels."""
#!/usr/bin/env python3

from curses import wrapper, curs_set, init_pair, color_pair
from curses import echo, noecho
from curses import COLOR_BLACK, COLOR_YELLOW, COLOR_CYAN
from locale import setlocale, LC_ALL
from subprocess import run
from channel import get_channels, add_channel, validate_link
from config import _HIGHLIGHT, _LINK_HANDLER, _BASE_YT_WATCH_URL
from scrape import parse_channels
from video_data import get_video_list

setlocale(LC_ALL, '')


def execute_action(stdscr, c, current, length):
    """Handle key presses."""
    if c == ord('q'):
        quit()
    elif c == ord('g'):
        current = 0
    elif c == ord('G'):
        current =  length - 1
    elif c == ord('j'):
        current += 1
        if current == length:
            current -= 1
    elif c == ord('k'):
        current -= 1
        if current == -1:
            current = 0
    elif c == ord('a'):
        load_add_channel_menu(stdscr)
    elif c == ord('r'):
        load_refresh_menu(stdscr)
    return current

def print_channel_info(stdscr, data, current):
    """Print channel info."""
    rows, cols = stdscr.getmaxyx()
    stdscr.addstr(0, 0, f'{"Channel":<{cols}}', color_pair(1) | _HIGHLIGHT)
    for i, c in enumerate(data):
        channel_str = f'{c:<{cols}}'
        if i == current:
            stdscr.addstr(i + 1, 0, channel_str, color_pair(2) | _HIGHLIGHT)
        else:
            stdscr.addstr(i + 1, 0, channel_str)
    col = int(cols / 3)
    s_bar = f'{"q: Quit":<{col}}{"a: Add channel":<{col}}{"r: Refresh":<{col}}'
    stdscr.addstr(rows - 2, 0, f'{s_bar:<{cols}}', color_pair(1) | _HIGHLIGHT)


def print_video_info(stdscr, data, current):
    """Print video info."""
    rows, cols = stdscr.getmaxyx()
    col = int(cols / 10)
    rem = int(cols - 3 * col)
    header = f'{"Date":>{col}}{"Time":>{col}}{" "*col}{"Title":<{rem}}'
    stdscr.addstr(0, 0, header, color_pair(1) | _HIGHLIGHT)
    for i, v in enumerate(data):
        s = f'{v["date"]:>{col}}{v["time"]:>{col}}{" "*col}{v["name"]:<{rem}}'
        if i == current:
            stdscr.addstr(i + 1, 0, s, color_pair(2) | _HIGHLIGHT)
        else:
            stdscr.addstr(i + 1, 0, s)
    stdscr.addstr(rows - 2, 0, f'{"":<{cols}}', color_pair(1) | _HIGHLIGHT)



def show_list(stdscr, print_func, data, current):
    """Print the entries for the given data."""
    stdscr.clear()
    print_func(stdscr, data, current)
    stdscr.refresh()


def load_refresh_menu(stdscr):
    """Refresh the video data."""
    rows, cols = stdscr.getmaxyx()
    status = f'{"Please wait..":<{cols}}'
    stdscr.addstr(rows - 2, 0, status, color_pair(1) | _HIGHLIGHT)
    stdscr.refresh()
    parse_channels()


def load_add_channel_menu(stdscr):
    """Show Add channel menu."""
    rows, cols = stdscr.getmaxyx()
    curs_set(True)
    echo()
    stdscr.addstr(rows - 2, 0, f'{" ":{cols}}', color_pair(1) | _HIGHLIGHT)
    stdscr.addstr(rows - 2, 0, f'channel url: ', color_pair(1) | _HIGHLIGHT)
    stdscr.refresh()
    url = (stdscr.getstr()).decode()
    if validate_link(url):
        if add_channel(url):
            text = f'{"channel added. press any key to continue...":<{cols}}'
        else:
            text = f'{"channel exists. press any key to continue...":<{cols}}'
    else:
        text = f'{"invalid link. press any key to continue...":<{cols}}'
    stdscr.addstr(rows - 2, 0, text, color_pair(1) | _HIGHLIGHT)
    stdscr.refresh()
    stdscr.getch()
    noecho()
    curs_set(False)
    pass


def load_video_list_menu(stdscr, channel):
    """Show the video list menu."""
    videos = get_video_list(channel)
    length = len(videos)
    current = 0
    show_list(stdscr, print_video_info, videos, current)

    while True:
        c = stdscr.getch()
        if c == ord('h'):
            return
        if c == ord('l'):
            video_url = _BASE_YT_WATCH_URL + videos[current]['id']
            process = _LINK_HANDLER + ' ' + video_url
            run([process], shell=True)
        else:
            current = execute_action(stdscr, c, current, length)
        show_list(stdscr, print_video_info, videos, current)


def load_home_menu(stdscr):
    """Show the home menu."""
    channels = get_channels()
    length = len(channels)
    current = 0
    show_list(stdscr, print_channel_info, channels, current)

    while True:
        c = stdscr.getch()
        if c == ord('l'):
            load_video_list_menu(stdscr, channels[current])
        else:
            current = execute_action(stdscr, c, current, length)
        show_list(stdscr, print_channel_info, channels, current)


def ncurses_defaults() -> None:
    """Set basic defaults for ncurses."""
    curs_set(False)
    init_pair(1, COLOR_YELLOW, COLOR_BLACK)
    init_pair(2, COLOR_CYAN, COLOR_BLACK)


def start_app(stdscr) -> None:
    """Function wrapper."""
    ncurses_defaults()
    load_home_menu(stdscr)


def main() -> None:
    """Start the program."""
    wrapper(start_app)


if __name__ == '__main__':
    main()



"""
Code to save and extract video info.

functions:
    save_video_info
    get_video_info
"""
#!/usr/bin/env python3

from config import _DATA_FILE
from storage import get_data, save_data


def save_video_info(parsed_data: list) -> None:
    """Save the video info.

    arguments:
        parsed_data

    return:
        None
    """
    save_data(_DATA_FILE, parsed_data)

def get_video_info() -> any:
    """Get the video info.

    return:
        video-info
    """
    return get_data(_DATA_FILE)


def get_video_list(channel: str) -> list:
    """Get the video list of channel.

    arguments:
        channel

    return:
        [video data]
    """
    video_list = get_video_info()
    return video_list[channel]


if __name__ == '__main__':
    print(get_video_info())



"""
Contains code to storage and retrieve data.

functions:
    save_data
    get_data
"""
#!/usr/bin/env python3

from os.path import isfile
from pickle import dump, load


def save_data(file_name: str, data: any) -> None:
    """Save data to file_name.

    arguments:
        file_name
        data

    return:
        None
    """
    with open(file_name, 'wb') as writer:
        dump(data, writer)


def get_data(file_name: str) -> any:
    """Extract data from file_name.

    arguments:
        file_name

    return:
        [data]
    """
    if not isfile(file_name):
        return []
    with open(file_name, 'rb') as reader:
        data = load(reader)
    return data


if __name__ == '__main__':
    pass



"""
Code for subscription list.

functions:
    add_channel
    get_channels
"""
#!/usr/bin/env python3

from os.path import isfile
from re import search
from config import _CHANNEL_FILE
from storage import get_data, save_data


def validate_link(url: str) -> bool:
    """Validate if a link given is valid or not.

    arguments:
        url

    return:
        None
    """
    match = search("^https://www.youtube.com/c/\w+/\w+$", url)
    if match == None:
        return False
    else:
        return True


def add_channel(url: str) -> bool:
    """Add a channel to the list.

    arguments:
        url - channel url

    return:
        None
    """
    channels = []
    if isfile(_CHANNEL_FILE):
        channels = get_data(_CHANNEL_FILE)
    channel = url.split('/')[-2]
    if channel not in channels:
        channels.append(channel)
        save_data(_CHANNEL_FILE, channels)
        return True
    return False


def get_channels() -> list:
    """Get the list of channels suscribed.

    return:
        [channels]
    """
    return get_data(_CHANNEL_FILE)


if __name__ == '__main__':
    print(get_channels())



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console :: Curses",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
    ],
    url="https://github.com/subrat-lima/view_utube",
    author="Subrat Lima",
    author_email="subrat.k.lima@protonmail.com",
    name='view_utube',
    version='0.0.2',
    description='browse latest videos of your favorite youtube channels from terminal!',
    package_dir={'':'src'},
    packages=find_packages(where='src'),
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["gazpacho"],
    entry_points={
        'console_scripts': [
            'utube=view_utube:main',
        ],
    },
)



"""
Code to scrape data.

functions:
    add_channel
    get_channels

constants:
    _CHANNEL_FILE
"""
#!/usr/bin/env python3

from json import loads
from os import remove
from os.path import isfile
from gazpacho import Soup
from channel import get_channels
from config import _BASE_DIR, _DATA_FILE
from config import _CHANNEL_VIDEO_START, _CHANNEL_VIDEO_END
from storage import get_data, save_data


def select_script(scripts):
    """Select the script with the maximum length."""
    max_len = len(scripts[0].text)
    index = 0

    for i in range(1, len(scripts)):
        l = len(scripts[i].text)
        if l > max_len:
            max_len = l
            index = i

    return scripts[index].text



def scrape_page(channel_id: str) -> None:
    """Scrape for json data from the given url.

    arguments:
        url

    return:
        None
    """
    url = _CHANNEL_VIDEO_START + channel_id + _CHANNEL_VIDEO_END
    soup = Soup.get(url)
    scripts = soup.find('body').find('script')
    script = select_script(scripts)
    elem = script.strip().split('\n')[0].strip(';')
    elem = loads(elem[elem.index('=')+1:])
    elem = elem['contents']['twoColumnBrowseResultsRenderer']['tabs'][1][
            'tabRenderer']['content']['sectionListRenderer']['contents'][
            0]['itemSectionRenderer']['contents'][0]['gridRenderer']['items']
    file_name = _BASE_DIR + channel_id + '.json'
    save_data(file_name, elem)


def filter_video_info(file_name: str) -> list:
    """Filter video info from json file.

    arguments:
        file_name

    return:
        [video_info]
    """
    json_content = get_data(file_name)
    videos_info = []
    for entry in json_content:
        video = {}
        data = entry['gridVideoRenderer']
        video['id'] = data['videoId']
        video['name'] = data['title']['runs'][0]['text']
        video['date'] = data['publishedTimeText']['simpleText']
        split_str = video['date'].split()
        if 'Streamed' in split_str:
            video['date'] = " ".join(split_str[1:])
        video['time'] = data['thumbnailOverlays'][0][
                'thumbnailOverlayTimeStatusRenderer']['text']['simpleText']
        videos_info.append(video)
    return videos_info


def parse_channels() -> None:
    """Parse channels and save video info.

    return:
        None
    """
    channels = get_channels()
    for channel in channels:
        file_name = _BASE_DIR + channel + '.json'
        if not isfile(file_name):
            scrape_page(channel)
    video_data = {}
    for channel in channels:
        file_name = _BASE_DIR + channel + '.json'
        video_data[channel] = filter_video_info(file_name)
    save_data(_DATA_FILE, video_data)
    for channel in channels:
        file_name = _BASE_DIR + channel + '.json'
        if isfile(file_name):
            remove(file_name)


if __name__ == '__main__':
    parse_channels()



"""Configuration file."""
#!/usr/bin/env python3

from os.path import expanduser
from curses import A_BOLD, A_STANDOUT

# program to handle the youtube video link
# change it to the program of your choice
_LINK_HANDLER = 'utlh'

# default file path
_BASE_DIR = f'{expanduser("~")}/.local/utube/'
_CHANNEL_FILE = _BASE_DIR + 'channels.txt'
_DATA_FILE = _BASE_DIR + 'data.txt'

# constants for youtube link path
_BASE_YT_URL = 'https://www.youtube.com/'
_CHANNEL_VIDEO_START = _BASE_YT_URL + 'c/'
_CHANNEL_VIDEO_END = '/videos'
_BASE_YT_WATCH_URL = _BASE_YT_URL + 'watch?v='

# constant for ncurses
_HIGHLIGHT = A_BOLD | A_STANDOUT
